#include <stdbool.h>
#include "stm32l0xx.h"
#include "hw_gpio.h"
#include "hw_msp.h"
#include "radio.h"
#include "vcom.h"
#include "timeServer.h"

static TimerEvent_t ledTimer;
static uint8_t* buffer = "Hello LoRa";

#define LORA_FREQUENCY 868000000
#define LORA_TX_POWER 14
#define LORA_BANDWIDTH 2//0
#define LORA_DATARATE 7//10
#define LORA_CODERATE 1
#define LORA_PREAMBLE_LEN 8

static void gpioCallback()
{
	BSP_LED_On(LED3);
	Radio.Send(buffer, 10);
}

static void ledTimerCallback()
{
	BSP_LED_Off(LED2);
}

void txDoneEventCallback()
{
	BSP_LED_Off(LED3);
    Radio.Rx(0);
}

void rxDoneEventCallback(uint8_t *payload, uint16_t size, int16_t rssi, int8_t snr)
{
	BSP_LED_On(LED2);
	TimerStart(&ledTimer);
	PRINTF("->");
	for(int i=0; i<size; i++)
		PRINTF("%c", payload[i]);
	PRINTF("\n");
}

int main(void)
{
	HAL_Init();
	SystemClock_Config();
	HW_Init();

	RadioEvents_t radioEvents;
    radioEvents.TxDone = txDoneEventCallback;
    radioEvents.RxDone = rxDoneEventCallback;
    Radio.Init(&radioEvents);

    Radio.SetChannel(LORA_FREQUENCY);
    Radio.SetTxConfig(MODEM_LORA, LORA_TX_POWER, 0, LORA_BANDWIDTH, LORA_DATARATE, LORA_CODERATE, LORA_PREAMBLE_LEN,
                      false, true, false, 0, 0, 3000000);

    Radio.SetRxConfig(MODEM_LORA, LORA_BANDWIDTH, LORA_DATARATE, LORA_CODERATE, 0, LORA_PREAMBLE_LEN,
                      1000, false, 0, true, false, 0, false, true);

    Radio.Rx(0);

    TimerInit(&ledTimer, ledTimerCallback);
    TimerSetValue(&ledTimer, 500);

    HW_GPIO_SetIrq(GPIOB, GPIO_PIN_2, 0, gpioCallback);
    BSP_PB_Init(BUTTON_USER, BUTTON_MODE_EXTI);

	for(;;);
}
